% 0----------------------------------------------------------------------0
% | Program transient.m                                                  |
% | Used to find properties of flow in the transient system.             |
% | Uses data from:                                                      |
% |     blowout.mat                                                      |
% |     initdata.mat                                                     |
% |     ss(...).mat                                                      |
% | Saves figures in:                                                    |
% |     V_surf_(...).eps                                                 |
% |     Q_0_surf_(...).eps                                               |
% |     p_V_(...).eps                                                    |
% |                                                                      |
% | Coded by:                                                            |
% |     Stud. Techn. Eivind Vrlstad,                                    |
% |     Institutt for Energi- og prosessteknikk, NTNU, 2011              |
% 0----------------------------------------------------------------------0

clear all
clc
tic

%O-----------------------------------------------------------------------O
%| Global constants                                                      |
%O-----------------------------------------------------------------------O

global g PI theta sinalpha D tol maxit steptime M M3 N ...
    oila_0 oilmu_0 oilK_0 oilT oilp_0 oilrho_0 ...
    seaT searho_0 ...
    riserH ...
    x dx dx1 dx2 dx3 dt H p_init rho_init ...
    steeleps steelE steele


tol = 1E-20;
maxit = 1000;

load('../blowout.mat')
load('../initdata.mat')
load('../ssD400/ssD400.mat')



%O-----------------------------------------------------------------------O
%| Riser properties                                                      |
%O-----------------------------------------------------------------------O

riserD = .4;            %Diameter [m]



%O-----------------------------------------------------------------------O
%| Well properties                                                       |
%O-----------------------------------------------------------------------O

wellD = .3;             %Diameter [m]



%O-----------------------------------------------------------------------O
%| Fluid properties                                                      |
%O-----------------------------------------------------------------------O

seaT = 283.15;          %Temperature of sea water [K]

oilT = 283.15;          %Temperature of oil [K]
oilp_0 = 101.325E3;     %Reference pressure [Pa]

p_atm = oilp_0;



% 0----------------------------------------------------------------------0
% | Defining the grid                                                    |
% | dx  . .  |  . .                                                      |
% | x  o o o | o o o                                                     |
% | m  1 2 3 | ... M                                                     |
% 0----------------------------------------------------------------------0

M3 = 3*M+1;

dx1 = wellL/M;
x1 = (1:M+1)'*dx1;

dx2 = wellH/M;
x2 = wellL+(2:M+1)'*dx2;

dx3 = riserH/M;
x3 = wellL+wellH+(2:M+1)'*dx3;



N_dt = 50*M;            %Number of time steps
t_0 = 0;                %Starting time [s]
t_end = 40;             %End time [s]
N = N_dt+1;             %Number of nodes in time
dt = (t_end-t_0)/N_dt;  %Time step [s]


x = [x1;x2;x3];
t = 0:dt:t_end;



%0-----------------------------------------------------------------------0
%| Setting constants that vary through the pipe                          |
%0-----------------------------------------------------------------------0

sinalpha = [zeros(M,1);ones(2*M,1)];
D = [ones(2*M,1)*wellD;ones(M+1,1)*riserD];
A_0 = pi/4*D.^2;
H = wellH+riserH-[zeros(M+1,1); (1:M)'*dx2; wellH+(1:M)'*dx3];
dx = [ones(M,1)*dx1; ones(M,1)*dx2; ones(M,1)*dx3];
theta = dx/(2*dt);


if dt*oila_0<=max([dx1 dx2 dx3])
    disp(['dt*a = ' num2str(dt*oila_0) ', dx = ' num2str(max([dx1 dx2 dx3])) '. Grid OK'])
else
    disp(['dt*a = ' num2str(dt*oila_0) ', dx = ' num2str(max([dx1 dx2 dx3])) '. Grid unstable'])
    break
end

%O-----------------------------------------------------------------------O
%| Initial values                                                        |
%O-----------------------------------------------------------------------O



[rho_riser_init p_riser_init] = rho_p_init(H((2*M+1):M3));

V_init = [V_blowout(1:2*M,wellD*10-1);zeros(M+1,1)];
rho_init = [rho_blowout(1:2*M,wellD*10-1);rho_riser_init];
p_init = [p_blowout(1:2*M,wellD*10-1);p_riser_init];


steptime = toc;
disp(['Initial values calculated. Time: ', num2str(steptime), ' s.'])

PI = PI(wellD*10-1);



%O-----------------------------------------------------------------------O
%| Valve properties                                                      |
%O-----------------------------------------------------------------------O

valvetime = 10;
valvep_end = 20E5;
dp = (p_init(M3,1)-valvep_end)/valvetime*dt;
valvep = p_init(M3,1):-dp:valvep_end;



%O-----------------------------------------------------------------------O
%| Initiating solver                                                     |
%O-----------------------------------------------------------------------O

p_new = p_init;
rho_new = rho_init;
V_new = V_init;


% Allocating space for the matrices stroing the last 4 loops' values of
% variables
rho_curr = rho_init*ones(1,4);
p_curr = p_init*ones(1,4);
V_curr = V_init*ones(1,4);
t_curr = zeros(M3,4);
x_curr = x*ones(1,4);
a_curr = sqrt(oilK_0./rho_curr./(1+oilK_0/steelE*D*ones(1,4)/steele));
A_curr = A_0*ones(1,4).*oilrho_0./rho_curr.*exp(oilK_0./a_curr.^2.*(1/oilrho_0-1./rho_curr));
Q_0_curr = A_curr.*V_curr;


% Allocating space for the result matrices
V_res = zeros(M3,(N-1)*50/M+1);
p_res = zeros(M3,(N-1)*50/M+1);
rho_res = zeros(M3,(N-1)*50/M+1);
t_res = zeros(M3,(N-1)*50/M+1);
x_res = x*ones(1,(N-1)*50/M+1);
Q_0_res = zeros(M3,(N-1)*50/M+1);

V_res(:,1) = V_init;
rho_res(:,1) = rho_init;
p_res(:,1) = p_init;
Q_0_res(:,1) = Q_0_curr(:,1);


Re_new = D.*abs(V_curr(:,1)).*rho_curr(:,1)/oilmu_0;
Re_new(Re_new<1E3) = 2E3;
f = (-1.8.*real(log10(6.9./Re_new + (steeleps./D/3.7).^1.11))).^-2;
for n=2:N
    tic
    
    
    
    % A,D: Left, B,E: Right
    V_A = V_curr(1:M3-1,1);
    V_B = V_curr(2:M3,1);
    V_D = V_curr(1:M3-1,3);
    V_E = V_curr(2:M3,3);
    
    % A,D: Left, B,E: Right
    rho_A = rho_curr(1:M3-1,1);
    rho_B = rho_curr(2:M3,1);
    rho_D = rho_curr(1:M3-1,3);
    rho_E = rho_curr(2:M3,3);
    
    a_R = a_curr(1:M3-1,1);
    a_S = a_curr(2:M3,1);
    
    % Interpolating left (R) values for V and rho at t-dt
    V_X = V_A*3/4 ...
        - V_D/4 ...
        - a_R/2;
    
    V_R = (V_X ...
        + sqrt(V_X.^2 ...
        + (3/2*a_R - theta) ...
        .* (V_A - V_D) ...
        + a_R.*V_D)).*A_curr(1:M3-1,1)./A_curr(2:M3,1);
    
    rho_R = 1./(1./rho_D ...
        - (1./rho_A - 1./rho_D) ...
        .* (theta./(V_R + a_R) - 3/2));
    
    % Interpolating right (S) values for V and rho at t-dt
    V_Y = 3/4*V_B ...
        - V_E/4 ...
        + a_S/2;
    
    V_S = (V_Y ...
        - sqrt(V_Y.^2 ...
        - (3/2*a_S - theta) ...
        .* (V_B - V_E) ...
        - a_S.*V_E)).*A_curr(2:M3,1)./A_curr(1:M3-1,1);
    
    rho_S = 1./(1./rho_E ...
        + (1./rho_B - 1./rho_E) ...
        .* (theta./(V_S - a_S) + 3/2));
    
    
    err = inf;
    it = 0;
    while err>tol && it<maxit
        rho_old = rho_new;
        
        % Calculating intermidiate variables
        B_R = a_R/oilK_0;
        B_S = a_S/oilK_0;
        R_R = dx.*a_R.*f(1:M3-1)./(2*D(1:M3-1)*oilK_0);
        R_S = dx.*a_S.*f(2:M3)./(2*D(2:M3)*oilK_0);
        
        % Calculating left (+) variables
        C_P = 1./rho_R ...
            - B_R.*abs(V_R) ...
            + g*dx.*sinalpha.*B_R ...%dx(1:M3-1,1).*sinalpha(1:M3-1,1)
            ./ (V_R+a_R);
        
        B_P = B_R ...
            + R_R.*abs(V_R) ...
            ./ (V_R+a_R);
        
        % Calculating right (-) variables
        C_M = 1./rho_S ...
            + B_S.*abs(V_S) ...
            + g*dx.*sinalpha.*B_S ...
            ./ (V_S-a_S);
        
        B_M = B_S ...
            - R_S.*abs(V_S) ...
            ./ (V_S-a_S);
        
        % Calculating V-values
        V_P = (C_M(2:M3-1) ...
            - C_P(1:M3-2)) ...
            ./ (B_M(2:M3-1) ...
            + B_P(1:M3-2));
        
        
        % Valve opening
        if t_curr(1,1) < valvetime-dt
            p_M3 = valvep(n);
            rho_M3 = oilrho_0*exp((p_M3-oilp_0)/oilK_0);
        else
            rho_M3 = rho_curr(M3,2);
        end
        
        
        %Calculating end values for V
        V_M3 = 1/B_P(M3-1)*(1/rho_M3-C_P(M3-1));
        
        
        rho_1_new = rho_curr(1,2);
        p_1_new = p_curr(1,2);
        p_R = searho_0*g*H(1)+oilp_0;
        err2 = inf;
        it2 = 0;
        while err2>tol && it2<=maxit
            rho_1_old = rho_1_new;
            p_1_old = p_1_new;
            
            p_1_new = (9*p_1_old+abs(p_R - A_0(1)/B_M(1)/PI*(C_M(1)-1/rho_1_old)))/10;%*exp(oilK/oila^2*(1/oilrho_0-1/rho_1_old))))/2;
            rho_1_new = oilrho_0*exp((p_1_new-oilp_0)/oilK_0);
            
            
            if it2 < 3;
                err2 = inf;
            else
                err2 = norm((rho_1_new-rho_1_old).^2);
            end
            
            it2 = it2+1;
        end
        
        
        V_1 = -1/B_M(1)*(1/rho_1_new-C_M(1));
        rho_1 = rho_1_new;
        p_1 = p_1_new;
        
        %Inserting new end values for V
        V_new = [V_1; V_P; V_M3];
        
        
        % Calculating rho-values
        rho_P = (B_P(1:M3-2) ...
            + B_M(2:M3-1)) ...
            ./ (C_P(1:M3-2).*B_M(2:M3-1) ...
            + C_M(2:M3-1).*B_P(1:M3-2));
        
        
        rho_new = [rho_1; rho_P; rho_M3];
        
        % Making shure that head velocity and density never falls below
        % initial, and making an end of the riser at 2M+1 if it does
        if V_new(2*M) < V_init(2*M,1) || rho_new(2*M) > rho_init(2*M,1)
            V_new(2*M) = V_init(2*M,1);
            rho_new(2*M) = rho_init(2*M);
            rho_new(2*M+1) = rho_init(2*M+1);
            V_new(2*M+1) = (C_M(2*M+2)-1/rho_new(2*M+1))./B_M(2*M+2);
        end
        
        %Calculating new values for p at n
        p_new = oilp_0+oilK_0*reallog(rho_new/oilrho_0);
        
        
        % Calculating new values for a and f
        a = sqrt(oilK_0./rho_new./(1+oilK_0/steelE*D/steele));
        Re_new = D.*abs(V_new).*rho_new/oilmu_0;
        Re_new(Re_new<1E3) = 2E3;
        f = (-1.8.*real(log10(6.9./Re_new + (steeleps./D/3.7).^1.11))).^-2;
        
        %Calculating error
        err = norm((rho_old-rho_new).^2);
        if it>maxit
            disp(['Warning: V does not converge in ' num2str(maxit) ' iterations in n = ' num2str(n)])
            break
        elseif it<3
            err = inf;
            it = it+1;
        else
            it = it+1;
        end
    end
    
    
    % Storing current data, pushing old data out
    rho_curr = [rho_new rho_curr(:,1:3)];
    p_curr = [p_new p_curr(:,1:3)];
    V_curr = [V_new V_curr(:,1:3)];
    t_curr = [t_curr(:,1)+dt t_curr(:,1:3)];
    a_curr = [sqrt(oilK_0./rho_curr(:,1)./(1+oilK_0/steelE*D/steele)) a_curr(:,1:3)];
    A_curr = [A_0.*oilrho_0./rho_curr(:,1).*exp(oilK_0./a_curr(:,1).^2.*(1/oilrho_0-1./rho_curr(:,1))) A_curr(:,1:3)];
    Q_0_curr = A_curr.*V_curr;
    
    %Combining and inserting into the answer matrices, each 10th time step
    if (n-1)*50/M == floor((n-1)*50/M)
        V_res(:,(n-1)*50/M+1) = V_new;
        rho_res(:,(n-1)*50/M+1) = rho_new;
        p_res(:,(n-1)*50/M+1) = p_new;
        t_res(:,(n-1)*50/M+1) = t_curr(:,1);
        x_res(:,(n-1)*50/M+1) = x_curr(:,1);
        Q_0_res(:,(n-1)*50/M+1) = Q_0_curr(:,1);
    end
    
    
    steptime(n) = toc;
    clc
    disp(['Time step ', num2str(n), ' of ', num2str(N), ', calculation time: ', num2str(steptime(n)), ' s.'])
    
end

clc
disp(['Total calculation time: ' num2str(sum(steptime)) ' s'])





x_trVt10s20bar = x_res(:,1);
t_trVt10s20bar = t_res(1,:);
V_trVt10s20bar = V_res([1 2*M 2*M+1 M3],:);
p_trVt10s20bar = p_res([1 2*M 2*M+1 M3],:);
Q_0_trVt10s20bar = Q_0_res([1 2*M 2*M+1 M3],:);
save('trVt10s20bar','x_trVt10s20bar','t_trVt10s20bar','V_trVt10s20bar','p_trVt10s20bar','Q_0_trVt10s20bar','PI')


figure(1)
clf
surf(x_res,t_res,V_res,'FaceColor','interp','EdgeColor','none','FaceLighting','phong')
title('Velocity')
xlabel('x [m]')
ylabel('t [s]')
zlabel('V [m/s]')
axis([0 max(x) 0 t_end])
view(50,50)
print -depsc2 ../plots/V_surf_trVt10s20bar


figure(2)
clf
surf(x_res,t_res,Q_0_res,'FaceColor','interp','EdgeColor','none','FaceLighting','phong')
title('Volumetric flow')
xlabel('x [m]')
ylabel('t [s]')
zlabel('[m^3/s]')
axis([0 max(x) 0 t_end])
view(50,30)
print -depsc2 ../plots/Q_0_surf_trVt10s20bar

figure(3)
clf
surf(x_res,t_res,p_res*1E-5,'FaceColor','interp','EdgeColor','none','FaceLighting','phong')
title('Pressure')
xlabel('x [m]')
ylabel('t [s]')
zlabel('[bar]')
axis([0 max(x) 0 t_end])
view(50,50)


figure(4)
clf
subplot(2,1,1)
hold on
plot(t_res(1,:),Q_0_res(1,:),'k:','LineWidth',1)
plot(t_res(1,:),Q_0_res(2*M,:),'k--','LineWidth',1)
plot(t_res(1,:),Q_0_res(M3,:),'k','LineWidth',1)
hold off
title('Volumetric flow Q_0')
xlabel('t [s]')
ylabel('Q [m^3/s]')
legend('Well', ...
    'Wellhead', ...
    'Top')

subplot(2,1,2)
hold on
plot(t_res(1,:),V_res(1,:)*1E-5,'k:','LineWidth',1)
plot(t_res(1,:),V_res(2*M,:)*1E-5,'k--','LineWidth',1)
plot(t_res(1,:),V_res(M3,:)*1E-5,'k','LineWidth',1)
hold off
title('Velocity')
xlabel('t [s]')
ylabel('V [m/s]')
print -depsc2 ../plots/p_V_trVt10s20bar
