% 0----------------------------------------------------------------------0
% | Program ssD300.m                                                     |
% | Used to find properties of flow in the system at steady-state        |
% | conditions and D = 300 mm.                                           |
% | Uses data from:                                                      |
% |     blowout.mat                                                      |
% |     initdata.mat                                                     |
% | Saves figures in:                                                    |
% |     V_V_diff_ss_D300.eps                                             |
% |     p_p_diff_ss_D300.eps                                             |
% |                                                                      |
% | Coded by:                                                            |
% |     Stud. Techn. Eivind Vrlstad,                                    |
% |     Institutt for Energi- og prosessteknikk, NTNU, 2011              |
% 0----------------------------------------------------------------------0

clear all
clc

tol = 1E-5;
maxit = 50;



% 0----------------------------------------------------------------------0
% | Loading data from blowout.m, stored in blowout.mat                   |
% 0----------------------------------------------------------------------0

load('../blowout')
load('../initdata')



% Properties of oil

oilp_0 = 101.3E3;


% Properties of well

wellD = .3;             % m

% Properties of riser

riserD = .3;


% Flow properties

p_r = searho_0*g*(wellH+riserH) + oilp_0;    % Reservoir pressure [Pa]


% Setting the grid

M3 = M*3+1;

dx1 = wellL/M;
x1 = (0:dx1:wellL)';

dx2 = wellH/M;
x2 = (wellL+dx2:dx2:(wellL+wellH))';

dx3 = riserH/M;
x3 = ((wellL+wellH)+dx3:dx3:(wellL+wellH+riserH))';

x = [x1;x2;x3];


% Setting constants that have different values through the pipe

sinalpha = [zeros(M,1);ones(2*M+1,1)];
D = [ones(2*M,1)*wellD;ones(M+1,1)*riserD];
A_0 = pi/4*D.^2;
H = sinalpha.*(x-x(M+1,1));


p_t = [20 15 10 5]*1E5;
p = zeros(M3,4);
V = zeros(M3,4);
rho = zeros(M3,4);


for i = 1:4
    % Setting boundary values
    
    p_P = p_t(1,i);              % Top pressure [Pa]
    rho_P = oilrho_0*exp((p_P-oilp_0)/oilK_0);    % Top density [kg/m^3]
    V_P = 5;                % First guess velocity [m/s]
    
    V_new = zeros(M3,1);
    rho_new = zeros(M3,1);
    p_new = zeros(M3,1);
    V_new(M3) = V_P;
    p_new(M3) = p_P;
    rho_new(M3) = rho_P;
    
    err = inf;
    it = 0;
    while err>=tol && it<maxit
        for m = M3-1:-1:1
            rho_S = rho_new(m+1);
            V_S = V_new(m+1)*A_0(m+1)/A_0(m);
            a_S = sqrt(oilK_0/rho_S/(1+oilK_0/steelE*D(m+1)/steele));
            s = 2*g/a_S^2;
            Re_D = abs(rho_S*V_S*D(m)/oilmu_0);
            f = (-1.8 * log10(6.9/Re_D ...
                + (steeleps/D(m)/3.7)^1.11))^-2;
            
            V_P = V_S ...
                / (exp(s*(H(m+1)-H(m))/2) ...
                + f*V_S*abs(V_S)*(x(m+1)-x(m))/(2*D(m)*a_S^2));
            V_new(m) = V_P;
            rho_P = 1/(1/rho_S ...
                - (g*(H(m+1)-H(m)) ...
                + f*V_P*abs(V_S)/2/D(m)*(x(m+1)-x(m))) ...
                /oilK_0);
            rho_new(m) = rho_P;
            p_new(m) = oilp_0 + oilK_0*reallog(rho_P/oilrho_0);
        end
        
        % Reassessing the velocity V_w, at the bottom of the well
        
        Q_w = PI(1,2) * (p_r - p_new(1,1));
        rho_w = rho_new(1,1);
        A_w = A_0(1,:)*oilrho_0./rho_w.*exp(oilK_0/oila_0^2*(1/oilrho_0-1./rho_w));
        V_w = Q_w/A_w*oilrho_0/rho_new(1,1);
        
        V_P = mean([V_w,V_new(1,1)]);
        rho_P = rho_new(1,1);
        
        V_new(1,1) = V_P;
        
        
        % Calculating back from bottom, with the new velocity
        
        for m = 2:M3
            rho_R = rho_P;
            V_R = V_P*A_0(m-1)/A_0(m);
            a_R = sqrt(oilK_0/rho_R/(1+oilK_0/steelE*D(m-1)/steele));
            s = 2*g/a_R^2;
            Re_D = abs(rho_R*V_R*D(m)/oilmu_0);
            f = (-1.8 * log10(6.9/Re_D ...
                + (steeleps/D(m)/3.7)^1.11))^-2;
            
            V_P = V_R ...
                / (exp(-s*(H(m)-H(m-1))/2) ...
                - f*V_R*abs(V_R)*(x(m)-x(m-1))/(2*D(m)*a_R^2));
            V_new(m) = V_P;
            rho_P = 1/(1/rho_R ...
                + (g*(H(m)-H(m-1)) ...
                + f*V_P*abs(V_R)/2/D(m)*(x(m)-x(m-1))) ...
                / oilK_0);
            rho_new(m) = rho_P;
            p_new(m) = oilp_0 + oilK_0*reallog(rho_P/oilrho_0);
        end
        
        p_P = p_t(i);
        V_P = V_new(M3);
        rho_P = oilrho_0*exp((p_P-oilp_0)/oilK_0);
        
        err = norm((p_new(M3,1)-p_t));
        
        it = it + 1;
        
        p_new(M3) = p_P;
        rho_new(M3) = rho_P;
    end
    
    p(:,i) = p_new;
    V(:,i) = V_new;
    rho(:,i) = rho_new;
    disp(['Case ' num2str(i) ' done'])
end


p_blowout = [p_blowout(1:2*M,wellD*10-1); ...
    p_blowout(2*M+1:M3,riserD*10-1)];
V_blowout = [V_blowout(1:2*M,wellD*10-1); ...
    V_blowout(2*M+1:M3,riserD*10-1)];
rho_blowout = [rho_blowout(1:2*M,wellD*10-1); ...
    rho_blowout(2*M+1:M3,riserD*10-1)];

V_ss_D300 = [V_blowout V];
rho_ss_D300 = [rho_blowout rho];
p_ss_D300 = [p_blowout p];
a_ss_D300 = sqrt(oilK_0./rho_ss_D300./(1+oilK_0/steelE.*D*ones(1,5)/steele));
A_ss_D300 = A_0*ones(1,5).*oilrho_0./rho_ss_D300.*exp(oilK_0./a_ss_D300.^2.*(1/oilrho_0-1./rho_ss_D300));
Q_ss_D300 = A_ss_D300.*V_ss_D300;
Q_ss_D300_0 = rho_ss_D300(1,:)/oilrho_0.*Q_ss_D300(1,:);

% Making matrices with the key data
p_ss_D300_well = [p_blowout(1) p(1,:)];
p_ss_D300_head = [p_blowout(2*M) p(2*M,:)];
p_ss_D300_top = [p_blowout(M3) p(M3,:)];
Q_ss_D300_well = [Q_blowout_0*oilrho_0./rho_blowout(1) Q_ss_D300(1,:)];
Q_ss_D300_head = [Q_blowout_0*oilrho_0./rho_blowout(2*M) Q_ss_D300(2*M,:)];
Q_ss_D300_top = [Q_blowout_0*oilrho_0./rho_blowout(M3) Q_ss_D300(M3,:)];

save('ssD300','V_ss_D300','Q_ss_D300', ...
    'rho_ss_D300','p_ss_D300', ...
    'p_ss_D300_well','p_ss_D300_head','p_ss_D300_top', ...
    'Q_ss_D300_well','Q_ss_D300_head','Q_ss_D300_top')





for i=1:5
    disp(['Case ' num2str(i)])
    disp(' ')
    disp(['D_well = ' num2str(wellD) ' m'])
    disp(['D_riser = ' num2str(riserD) ' m'])
    disp(['p_well = ' num2str(p_ss_D300(1,i)/1E5) ' bar'])
    disp(['p_head = ' num2str(p_ss_D300(2*M,i)/1E5) ' bar'])
    disp(['p_top = ' num2str(p_ss_D300(M3,i)/1E5) ' bar'])
    disp(['Q_0 = ' num2str(Q_ss_D300_0(i)) ' m^3/s'])
    disp(['V_top = ' num2str(V_ss_D300(M3,i)) ' m/s'])
    disp(' ')
    disp(' ')
end


p_w = p_ss_D300(1,:);
p_hydrostat = ones(M3,1)*p_w ...
    - oilrho_0*g*H*ones(1,5);

V_bernoulli = sqrt((ones(M3,1)*Q_ss_D300_0./(A_0*ones(1,5))).^2 ...
    - 2*g*H*ones(1,5) ...
    + ones(M3,1)*p_w*2/oilrho_0 ...
    - 2*p_hydrostat/oilrho_0);

p_diff = (p_hydrostat-p_ss_D300)/1E5;
V_diff = (V_bernoulli-V_ss_D300)./V_ss_D300*100;


figure(3)
clf
subplot(2,1,1)
hold on
plot(x,p_ss_D300(:,1)/1E5,'k','LineWidth',2)
plot(x,p_ss_D300(:,2)/1E5,'k:','LineWidth',1)
plot(x,p_ss_D300(:,3)/1E5,'k-.','LineWidth',1)
plot(x,p_ss_D300(:,4)/1E5,'k--','LineWidth',1)
plot(x,p_ss_D300(:,5)/1E5,'k','LineWidth',1)
hold off
title('Pressure')
xlabel('L [m]')
ylabel('p [bar]')
legend(['p_{top} = ' num2str(p_ss_D300(M3,1)/1E5) ' bar'], ...
    ['p_{top} = ' num2str(p_ss_D300(M3,2)/1E5) ' bar'], ...
    ['p_{top} = ' num2str(p_ss_D300(M3,3)/1E5) ' bar'], ...
    ['p_{top} = ' num2str(p_ss_D300(M3,4)/1E5) ' bar'], ...
    ['p_{top} = ' num2str(p_ss_D300(M3,5)/1E5) ' bar'])

subplot(2,1,2)
hold on
plot(x,p_diff(:,1),'k','LineWidth',2)
plot(x,p_diff(:,2),'k:','LineWidth',1)
plot(x,p_diff(:,3),'k-.','LineWidth',1)
plot(x,p_diff(:,4),'k--','LineWidth',1)
plot(x,p_diff(:,5),'k','LineWidth',1)
hold off
title('Pressure, Difference from Hydrostatic')
xlabel('L [m]')
ylabel('p_{diff} [bar]')

print -deps ../plots/V_V_diff_ss_D300


figure(4)
clf
subplot(2,1,1)
hold on
plot(x,V_ss_D300(:,1),'k','LineWidth',2)
plot(x,V_ss_D300(:,2),'k:','LineWidth',1)
plot(x,V_ss_D300(:,3),'k-.','LineWidth',1)
plot(x,V_ss_D300(:,4),'k--','LineWidth',1)
plot(x,V_ss_D300(:,5),'k','LineWidth',1)
hold off
title('Velocity')
xlabel('L [m]')
ylabel('V [m/s]')
legend(['p_t = ' num2str(p_ss_D300(M3,1)/1E5) ' bar'], ...
    ['p_t = ' num2str(p_ss_D300(M3,2)/1E5) ' bar'], ...
    ['p_t = ' num2str(p_ss_D300(M3,3)/1E5) ' bar'], ...
    ['p_t = ' num2str(p_ss_D300(M3,4)/1E5) ' bar'], ...
    ['p_t = ' num2str(p_ss_D300(M3,5)/1E5) ' bar'])

subplot(2,1,2)
hold on
plot(x,V_diff(:,1),'k','LineWidth',2)
plot(x,V_diff(:,2),'k:','LineWidth',1)
plot(x,V_diff(:,3),'k-.','LineWidth',1)
plot(x,V_diff(:,4),'k--','LineWidth',1)
plot(x,V_diff(:,5),'k','LineWidth',1)
hold off
title('Velocity, Difference from Bernoulli')
xlabel('L [m]')
ylabel('V_{diff} [%]')

print -deps ../plots/p_p_diff_ss_D300