% 0----------------------------------------------------------------------0
% | Program blowout.m                                                    |
% | Used to find properties of flow in the system at blowout conditions. |
% | Uses data in:                                                        |
% |     initdata.mat                                                     |
% | Saves results in:                                                    |
% |     blowout.mat                                                      |
% | Saves figures in:                                                    |
% |     p_p_diff_blowout.eps                                             |
% |     V_V_diff_blowout.eps                                             |
% |                                                                      |
% | Coded by:                                                            |
% |     Stud. Techn. Eivind Vrlstad,                                    |
% |     Institutt for Energi- og prosessteknikk, NTNU, 2011              |
% 0----------------------------------------------------------------------0

clear all
clc

tol = 1E-14;
maxit = 1000;

load('initdata')



% 0----------------------------------------------------------------------0
% | Properties of water                                                  |
% 0----------------------------------------------------------------------0




% 0----------------------------------------------------------------------0
% | Properties of oil                                                    |
% 0----------------------------------------------------------------------0

oilp_0 = 101.3E3;



% 0----------------------------------------------------------------------0
% | Various flow properties                                              |
% 0----------------------------------------------------------------------0

Q_bbl = 68000; % bbl/day
Q_0 = bbl_to_m3(Q_bbl)/3600/24; % m^3/s
m = Q_0 * oilrho_0; % kg/s
p_r = searho_0*g*(wellH+riserH) + oilp_0;    % Reservoir pressure [Pa]
p_h = searho_0*g*riserH + oilp_0;            % Wellhead pressure [Pa]



%0-----------------------------------------------------------------------0
%| Defining the grid                                                     |
%0-----------------------------------------------------------------------0

M3 = M*3+1;

dx1 = wellL/M;
x1 = (0:dx1:wellL)';

dx2 = wellH/M;
x2 = ((wellL+dx2):dx2:(wellL+wellH))';

dx3 = riserH/M;
x3 = ((wellL+wellH+dx3):dx3:(wellL+wellH+riserH))';

x = [x1;x2;x3];



%0-----------------------------------------------------------------------0
%| Setting constants that vary through the pipe                          |
%0-----------------------------------------------------------------------0

sinalpha = [zeros(M,1);ones(2*M+1,1)]; 
D = [.2 .3 .4 .5];
A_0 = pi/4*D.^2;
H = [zeros(M+1,1);(1:M)'*dx2;(M+1:2*M)'*dx3];



%0-----------------------------------------------------------------------0
%| Setting initial values                                                |
%0-----------------------------------------------------------------------0

p_P = p_h;
rho_P = oilrho_0*exp((p_P-oilp_0)/oilK_0);
A_P = A_0*oilrho_0./rho_P.*exp(oilK_0/oila_0^2*(1/oilrho_0-1./rho_P));
Q_P = Q_0*oilrho_0./rho_P;
V_P = Q_P./A_P;


%0-----------------------------------------------------------------------0
%| Defining the grid                                                     |
%0-----------------------------------------------------------------------0

V = zeros(M3,4);
rho = zeros(M3,4);
p = zeros(M3,4);
V(2*M,:) = V_P;
p(2*M,:) = p_P;
rho(2*M,:) = rho_P;



%0-----------------------------------------------------------------------0
%| Running sequence from sea bed to bottom of well (upstream)            |
%0-----------------------------------------------------------------------0

for m = 2*M-1:-1:1
        rho_S = rho_P;
        V_S = V_P;
        a_S = sqrt(oilK_0./rho_S./(1+oilK_0/steelE*D/steele));
        s = 2*g./a_S.^2;
        Re_D = abs(rho_S.*V_S.*D/oilmu_0);
        f = (-1.8 * log10(6.9./Re_D ...
            + (steeleps./D/3.7).^1.11)).^-2;
    
        V_P = V_S ...
            ./(exp(s*(H(m+1,:)-H(m,:))/2) ...
            + f.*V_S.*abs(V_S)*(x(m+1,:)-x(m,:))./(2*D.*a_S.^2));
        V(m,:) = V_P;
        rho_P = 1./(1./rho_S ...
            - (g*(H(m+1,:)-H(m,:)) ...
            + f.*V_P.*abs(V_S)/2./D.*(x(m+1,:)-x(m,:))) ...
            / oilK_0);
        rho(m,:) = rho_P;
        p(m,:) = oilp_0 + oilK_0*reallog(rho_P/oilrho_0);
end


V_P = V(2*M,:);
p_P = p(2*M,:);
rho_P = rho(2*M,:);

%0-----------------------------------------------------------------------0
%| Running sequence from sea bed to top of riser (downstream)            |
%0-----------------------------------------------------------------------0

for m = 2*M+1:M3
    rho_R = rho_P;
    V_R = V_P;
    a_R = sqrt(oilK_0./rho_R./(1+oilK_0/steelE*D/steele));
    s = 2*g./a_R.^2;
    Re_D = abs(rho_R.*V_R.*D/oilmu_0);
    f = (-1.8 * log10(6.9./Re_D ...
        + (steeleps./D/3.7).^1.11)).^-2;
    
    V_P = V_R ... 
        ./(exp(-s*(H(m,:)-H(m-1,:))/2) ...
        - f.*V_R.*abs(V_R)*(x(m,:)-x(m-1,:))./(2*D.*a_R.^2));
    V(m,:) = V_P;
    rho_P = 1./(1./rho_R ...
        + (g*(H(m,:)-H(m-1,:)) ...
        + f.*V_P.*abs(V_R)/2./D.*(x(m,:)-x(m-1,:))) ...
        / oilK_0);
    rho(m,:) = rho_P;
    p(m,:) = oilp_0 + oilK_0*reallog(rho_P/oilrho_0);
end



%0-----------------------------------------------------------------------0
%| Finding p_w and PI for the four cases;                                |
%| 1. Q1 D1                                                              |
%| 2. Q1 D2                                                              |
%| 3. Q2 D1                                                              |
%| 4. Q2 D2                                                              |
%0-----------------------------------------------------------------------0

p_w = p(1,:);           % Well pressure [Pa]
PI = Q_0./(p_r-p_w);      % Productivity index [m^3/Pa s]
F_b = p(M3,:)*pi/4.*D.^2;



%0-----------------------------------------------------------------------0
%| Displaying results on-screen                                          |
%0-----------------------------------------------------------------------0


disp('Case 0')
disp(' ')
disp('V_h = 0 m/s')
disp(['p_w = ' num2str(p_r/1E5) ' bar'])
disp(['p_t = ' num2str((searho_0-oilrho_0)*g*(riserH)/1E5) ' bar'])
disp('Q_0 = 0 m^3/s')
disp('PI = 0 m^3/bar s')
disp('Q_US = 0 bbl/d')
disp('PI_US = 0 bbl/psi d')
% disp(['F_buoy = ' num2str((searho_0-oilrho_0)*g*(riserH)*pi/4*riserD.^2/1E3) ' kN'])
disp(' ')
disp(' ')


for i=1:4
    disp(['Case ' num2str(i)])
    disp(' ')
    disp(['p_well = ' num2str(p(1,i)/1E5) ' bar'])
    disp(['p_head = ' num2str(p(2*M,i)/1E5) ' bar'])
    disp(['p_top = ' num2str(p(M3,i)/1E5) ' bar'])
    disp(['V_well = ' num2str(V(1,i)) ' m/s'])
    disp(['V_head = ' num2str(V(2*M,i)) ' m/s'])
    disp(['V_top = ' num2str(V(M3,i)) ' m/s'])
    disp(' ')
    disp(['D = ' num2str(D(i)) ' m'])
    disp(['Q_0 = ' num2str(Q_0) ' m^3/s'])
    disp(['PI = ' num2str(PI(i)*1E5) ' m^3/bar s'])
    disp(['Q_US = ' num2str(Q_bbl) ' bbl/d'])
    disp(['F_buoy = ' num2str(F_b(i)/1E3) ' kN'])
    disp(' ')
    disp(' ')
end


p_hydrostat = ones(M3,1)*p_w ...
    - oilrho_0*g*(H*ones(1,4));

V_bernoulli = sqrt(ones(M3,1)*(Q_0./A_0).^2 ...
    - 2*g.*H*ones(1,4) ...
    + ones(M3,1)*p_w*2/oilrho_0 ...
    - 2*p_hydrostat/oilrho_0);

p_diff_h = (p_hydrostat-p)/1E5;
V_diff_b = (V_bernoulli-V)./V*100;

x_blowout = x;
V_blowout = V;
rho_blowout = rho;
p_blowout = p;
Q_blowout_0 = Q_0;



%0-----------------------------------------------------------------------0
%| Saving results in blowout.mat                                         |
%0-----------------------------------------------------------------------0

save('blowout','x_blowout','V_blowout','rho_blowout','p_blowout','Q_blowout_0','PI')



%0-----------------------------------------------------------------------0
%| Plotting pressure results and saving to p_p_diff_blowout.eps          |
%0-----------------------------------------------------------------------0

figure(1)
subplot(2,1,1); plot(x,p(:,1)/1E5,'k', ...
    x,p(:,2)/1E5,'k--', ...
    x,p(:,3)/1E5,'k:', ...
    x,p(:,4)/1E5,'k-.')
title('Pressure')
xlabel('L [m]')
ylabel('p [bar]')
legend(['D = ' num2str(D(1,1)*1E3) ' mm'], ...
    ['D = ' num2str(D(1,2)*1E3) ' mm'], ...
    ['D = ' num2str(D(1,3)*1E3) ' mm'], ...
    ['D = ' num2str(D(1,4)*1E3) ' mm'])

subplot(2,1,2); plot(x,p_diff_h(:,1),'k', ...
    x,p_diff_h(:,2),'k--', ...
    x,p_diff_h(:,3),'k:', ...
    x,p_diff_h(:,4),'k-.')
title('Pressure, Difference From Hydrostatic')
xlabel('L [m]')
ylabel('p_{diff} [bar]')
print -deps plots/p_p_diff_blowout



%0-----------------------------------------------------------------------0
%| Plotting velocity results and saving to V_V_diff_blowout.eps          |
%0-----------------------------------------------------------------------0

figure(2)
subplot(2,1,1); plot(x,V(:,1),'k', ...
    x,V(:,2),'k--', ...
    x,V(:,3),'k:', ...
    x,V(:,4),'k-.')
title('Velocity')
xlabel('L [m]')
ylabel('V [m/s]')
legend(['D = ' num2str(D(1,1)*1E3) ' mm'], ...
    ['D = ' num2str(D(1,2)*1E3) ' mm'], ...
    ['D = ' num2str(D(1,3)*1E3) ' mm'], ...
    ['D = ' num2str(D(1,4)*1E3) ' mm'])


subplot(2,1,2); plot(x,V_diff_b(:,1),'k', ...
    x,V_diff_b(:,2),'k--', ...
    x,V_diff_b(:,3),'k:', ...
    x,V_diff_b(:,4),'k-.')
title('Velocity, Difference from Frictionless Bernoulli')
xlabel('L [m]')
ylabel('V_{diff} [%]')
print -deps plots/V_V_diff_blowout
